"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const load_features_1 = require("./load-features");
const console_reporter_1 = require("./console-reporter");
const backoff_1 = require("backoff");
const cucumber_messages_1 = require("cucumber-messages");
const replaceStoragePlaceholders_1 = require("./replaceStoragePlaceholders");
const retryConfiguration_1 = require("./retryConfiguration");
const allSuccess = (r, result) => (result.success ? r : false);
exports.afterRx = /^I am run after the "([^"]+)" feature$/;
class FeatureRunner {
    constructor(world, options) {
        this.stepRunners = [];
        this.cleaners = [];
        this.world = world;
        this.featuresDir = options.dir;
        this.reporters = options.reporters || [new console_reporter_1.ConsoleReporter()];
        this.store = options.store || {};
        this.retry = options.retry === undefined ? true : options.retry;
    }
    addStepRunners(runners) {
        this.stepRunners.push(...runners);
        return this;
    }
    cleanup(fn) {
        this.cleaners.push(fn);
    }
    async progress(type, info) {
        await Promise.all(this.reporters.map(reporter => reporter.progress(type, info)));
    }
    async run() {
        const features = await load_features_1.fromDirectory(this.featuresDir);
        const startRun = Date.now();
        const featureResults = [];
        await features.reduce((promise, feature) => promise.then(async () => {
            let skip = false;
            if (feature.dependsOn.length) {
                const depNames = feature.dependsOn.map(({ name }) => name);
                const dependendRuns = featureResults.filter(({ feature: { name } }) => depNames.includes(name));
                skip = !dependendRuns.reduce((allSucceeded, dep) => {
                    if (!allSucceeded)
                        return allSucceeded;
                    return dep.success;
                }, true);
            }
            if (skip) {
                featureResults.push({
                    feature: {
                        ...feature,
                        skip: true,
                    },
                    scenarioResults: [],
                    success: false,
                });
            }
            else {
                featureResults.push(await this.runFeature(feature));
            }
        }), Promise.resolve());
        const result = {
            success: featureResults.reduce(allSuccess, true),
            runTime: Date.now() - startRun,
            featureResults,
            store: this.store,
        };
        await Promise.all(this.reporters.map(reporter => reporter.report(result)));
        await this.cleaners.reduce((promise, cleaner) => promise.then(async () => cleaner(this).then(res => this.progress('cleaner', res))), Promise.resolve());
        return result;
    }
    async runFeature(feature) {
        var _a;
        await this.progress('feature', `${feature.name}`);
        if (feature.skip) {
            return {
                feature,
                success: true,
                scenarioResults: [],
            };
        }
        const startRun = Date.now();
        const scenarioResults = [];
        const flightRecorder = {
            flags: {},
            settings: {},
        };
        await ((_a = feature === null || feature === void 0 ? void 0 : feature.children) === null || _a === void 0 ? void 0 : _a.reduce((promise, scenario) => promise.then(async () => {
            var _a, _b, _c, _d, _e, _f;
            if (scenarioResults.length &&
                !scenarioResults[scenarioResults.length - 1].success) {
                scenarioResults.push({
                    success: false,
                    scenario: scenario,
                    tries: 0,
                    stepResults: [],
                    skipped: true,
                    retryConfiguration: retryConfiguration_1.retryConfiguration(scenario),
                });
                return;
            }
            if ((_b = (_a = scenario.scenario) === null || _a === void 0 ? void 0 : _a.examples) === null || _b === void 0 ? void 0 : _b.length) {
                let scenarioOutline = scenario.scenario;
                const example = (_c = scenarioOutline.examples) === null || _c === void 0 ? void 0 : _c[0];
                if (example) {
                    const header = (_e = (_d = example.tableHeader) === null || _d === void 0 ? void 0 : _d.cells) === null || _e === void 0 ? void 0 : _e.map(({ value }) => value);
                    await ((_f = example.tableBody) === null || _f === void 0 ? void 0 : _f.reduce((promise, example) => promise.then(async () => {
                        var _a, _b, _c, _d;
                        const values = (_a = example === null || example === void 0 ? void 0 : example.cells) === null || _a === void 0 ? void 0 : _a.map(({ value }) => value);
                        const replace = (str) => header === null || header === void 0 ? void 0 : header.reduce((str, _, k) => { var _a; return str === null || str === void 0 ? void 0 : str.replace(`<${header[k]}>`, (_a = values === null || values === void 0 ? void 0 : values[k]) !== null && _a !== void 0 ? _a : ''); }, str);
                        const s = {
                            keyword: 'Scenario',
                            name: `${(_b = scenario.scenario) === null || _b === void 0 ? void 0 : _b.name} (${values === null || values === void 0 ? void 0 : values.join(',')})`,
                            steps: (_d = (_c = scenario.scenario) === null || _c === void 0 ? void 0 : _c.steps) === null || _d === void 0 ? void 0 : _d.map(step => ({
                                ...step,
                                text: replace(step.text || ''),
                                docString: step.docString
                                    ? {
                                        ...step.docString,
                                        content: replace(step.docString.content || ''),
                                    }
                                    : undefined,
                            })),
                        };
                        if (this.retry) {
                            scenarioResults.push(await this.retryScenario(s, flightRecorder));
                        }
                        else {
                            scenarioResults.push(await this.runScenario(s, flightRecorder));
                        }
                    }), Promise.resolve()));
                }
            }
            else {
                const s = scenario.scenario || scenario.background;
                if (s) {
                    if (this.retry) {
                        scenarioResults.push(await this.retryScenario(s, flightRecorder));
                    }
                    else {
                        scenarioResults.push(await this.runScenario(s, flightRecorder));
                    }
                }
            }
        }), Promise.resolve()));
        return {
            success: scenarioResults.reduce(allSuccess, true),
            runTime: Date.now() - startRun,
            feature,
            scenarioResults,
        };
    }
    async retryScenario(scenario, feature) {
        return new Promise(async (resolve) => {
            let lastResult = await this.runScenario(scenario, feature);
            if (lastResult.success) {
                return resolve(lastResult);
            }
            const cfg = retryConfiguration_1.retryConfiguration(scenario);
            const b = backoff_1.exponential({
                randomisationFactor: 0,
                initialDelay: cfg.initialDelay,
                maxDelay: cfg.maxDelay,
            });
            b.failAfter(cfg.failAfter);
            b.on('ready', async (num) => {
                const r = await this.runScenario(scenario, feature);
                lastResult = {
                    ...r,
                    tries: num + 1,
                };
                if (lastResult.success) {
                    return resolve(lastResult);
                }
                await this.progress('retry', `${scenario.name}`);
                b.backoff();
            });
            b.on('fail', () => {
                resolve(lastResult);
            });
            b.backoff();
        });
    }
    async runScenario(scenario, feature) {
        var _a;
        await this.progress(scenario instanceof cucumber_messages_1.messages.GherkinDocument.Feature.Background
            ? 'background'
            : 'scenario', `${scenario.name}`);
        const startRun = Date.now();
        const stepResults = [];
        let abort = false;
        await ((_a = scenario === null || scenario === void 0 ? void 0 : scenario.steps) === null || _a === void 0 ? void 0 : _a.reduce((promise, step) => promise
            .then(async () => {
            if (abort) {
                stepResults.push({
                    skipped: true,
                    success: false,
                    step: {
                        ...step,
                        interpolatedText: `${step.text}`,
                    },
                });
            }
            else {
                stepResults.push(await this.runStep(step, feature));
            }
        })
            .catch(async (error) => {
            await this.progress('step error', error);
            stepResults.push({
                success: false,
                step: {
                    ...step,
                    interpolatedText: `${step.text}`,
                },
                error,
                skipped: false,
            });
            abort = true;
        }), Promise.resolve()));
        return {
            success: stepResults.reduce(allSuccess, true),
            runTime: Date.now() - startRun,
            scenario,
            stepResults,
            tries: 1,
            skipped: false,
            retryConfiguration: retryConfiguration_1.retryConfiguration(scenario),
        };
    }
    async runStep(step, feature) {
        await this.progress('step', `${step.text}`);
        const r = replaceStoragePlaceholders_1.replaceStoragePlaceholders({
            ...this.world,
            ...this.store,
        });
        const interpolatedStep = {
            ...step,
            interpolatedText: r(`${step.text}`),
            interpolatedArgument: step.docString
                ? r(`${step.docString.content}`)
                : undefined,
        };
        if (exports.afterRx.test(`${step.text}`)) {
            return {
                success: true,
                step: interpolatedStep,
                skipped: false,
            };
        }
        const matchedRunner = this.stepRunners.reduce((matchedRunner, runner) => {
            if (matchedRunner) {
                return matchedRunner;
            }
            const r = runner(interpolatedStep);
            if (r) {
                return r;
            }
            return undefined;
        }, undefined);
        if (!matchedRunner) {
            throw new StepRunnerNotDefinedError(interpolatedStep);
        }
        const startRun = Date.now();
        const result = await matchedRunner(this, feature);
        return {
            success: true,
            runTime: Date.now() - startRun,
            step: interpolatedStep,
            result,
            skipped: false,
        };
    }
}
exports.FeatureRunner = FeatureRunner;
class StepRunnerNotDefinedError extends Error {
    constructor(step) {
        super('No runner defined for this step!');
        this.step = step;
        this.name = StepRunnerNotDefinedError.name;
        Error.captureStackTrace(this, StepRunnerNotDefinedError);
        Object.setPrototypeOf(this, StepRunnerNotDefinedError.prototype);
    }
}
exports.StepRunnerNotDefinedError = StepRunnerNotDefinedError;
class RetryError extends Error {
    constructor(step) {
        super('Retrying step failed!');
        this.step = step;
        this.name = RetryError.name;
        Error.captureStackTrace(this, RetryError);
        Object.setPrototypeOf(this, RetryError.prototype);
    }
}
exports.RetryError = RetryError;
class StoreKeyUndefinedError extends Error {
    constructor(keys, store) {
        super(`"${keys.join('"')}" is not defined in the store!`);
        this.keys = keys;
        this.store = store;
        this.name = StoreKeyUndefinedError.name;
        Error.captureStackTrace(this, StoreKeyUndefinedError);
        Object.setPrototypeOf(this, StoreKeyUndefinedError.prototype);
    }
}
exports.StoreKeyUndefinedError = StoreKeyUndefinedError;
