"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const regexMatcher_1 = require("../lib/regexMatcher");
const chai = require("chai");
const chai_1 = require("chai");
const jsonata = require("jsonata");
const appSyncClient_1 = require("../lib/appsync/appSyncClient");
const queryWithIAM_1 = require("../lib/appsync/queryWithIAM");
const subscribe_1 = require("../lib/appsync/subscribe");
const queryWithApiKey_1 = require("../lib/appsync/queryWithApiKey");
const chaiSubset = require("chai-subset");
chai.use(chaiSubset);
exports.appSyncBeforeAll = async (runner) => {
    runner.store.appSyncClient = appSyncClient_1.AppSyncClient();
    return runner.store.appSyncClient;
};
exports.appSyncAfterAll = async (runner) => {
    Object.keys(runner.store.appSyncClient.subscriptions).forEach(id => runner.store.appSyncClient.subscriptions[id].disconnect());
};
const getQuery = (store, client, userId) => {
    if (client.authorization === 'API_KEY') {
        if (userId) {
            throw new Error('API_KEY authorization does not support user argument!');
        }
        return queryWithApiKey_1.queryWithApiKey(client.apiKey, client.endpoint);
    }
    else {
        const prefix = userId ? `cognito:${userId}` : `cognito`;
        const { [`${prefix}:AccessKeyId`]: AccessKeyId, [`${prefix}:SecretKey`]: SecretKey, [`${prefix}:SessionToken`]: SessionToken, } = store;
        return queryWithIAM_1.queryWithIAM(AccessKeyId, SecretKey, SessionToken, client.endpoint);
    }
};
exports.appSyncStepRunners = () => [
    regexMatcher_1.regexMatcher(/^the GQL endpoint is "([^"]+)"$/)(async ([endpoint], _, runner) => {
        const { appSyncClient: client } = runner.store;
        client.endpoint = endpoint;
    }),
    regexMatcher_1.regexMatcher(/^I execute this GQL query(?: as "([^"]+)")?$/)(async ([userId], step, runner) => {
        const { appSyncClient: client } = runner.store;
        if (!step.interpolatedArgument) {
            throw new Error('Must provide argument!');
        }
        const q = step.interpolatedArgument.replace(/\n\s*/g, ' ');
        await runner.progress('GQL>', q);
        const query = getQuery(runner.store, client, userId);
        const { result, operation, selection } = await query(q, client.variables);
        client.variables = {};
        client.response = result;
        client.operation = operation;
        client.selection = selection;
        await runner.progress('<GQL', JSON.stringify(result));
        return [q, result];
    }),
    regexMatcher_1.regexMatcher(/^the GQL query result should not contain errors$/)(async (_, __, runner) => {
        const { appSyncClient: client } = runner.store;
        chai_1.expect(client.response).to.not.have.property('errors');
    }),
    regexMatcher_1.regexMatcher(/^the GQL query result should contain this error$/)(async (_, step, runner) => {
        const { appSyncClient: client } = runner.store;
        if (!step.interpolatedArgument) {
            throw new Error('Must provide argument!');
        }
        chai_1.expect(client.response).to.have.property('errors');
        chai_1.expect(client.response.errors).to.containSubset([
            JSON.parse(step.interpolatedArgument),
        ]);
    }),
    regexMatcher_1.regexMatcher(/^(?:"([^"]+)" of )?the GQL response should (equal|match) this JSON$/)(async ([exp, equalOrMatch], step, runner) => {
        const { appSyncClient: client } = runner.store;
        if (!step.interpolatedArgument) {
            throw new Error('Must provide argument!');
        }
        const j = JSON.parse(step.interpolatedArgument);
        const result = client.response;
        const fragment = exp ? jsonata(exp).evaluate(result) : result;
        if (equalOrMatch === 'match') {
            chai_1.expect(fragment).to.containSubset(j);
        }
        else {
            chai_1.expect(fragment).to.deep.equal(j);
        }
        return result;
    }),
    regexMatcher_1.regexMatcher(/^"([^"]+)" of the GQL response should be (true|false)$/)(async ([exp, expected], _, runner) => {
        const { appSyncClient: client } = runner.store;
        const e = jsonata(exp);
        const v = e.evaluate(client.response);
        chai_1.expect(v).to.equal(expected === 'true');
        return v;
    }),
    regexMatcher_1.regexMatcher(/^"([^"]+)" of the GQL response should contain "([^"]+)"$/)(async ([exp, expected], _, runner) => {
        const { appSyncClient: client } = runner.store;
        const e = jsonata(exp);
        const v = e.evaluate(client.response);
        chai_1.expect(v).to.contain(expected);
        return v;
    }),
    regexMatcher_1.regexMatcher(/^I store "([^"]+)" of the GQL response as "([^"]+)"$/)(async ([expression, storeName], _, runner) => {
        const { appSyncClient: client } = runner.store;
        chai_1.expect(client.response).to.have.property('data');
        chai_1.expect(client.response.data).to.have.property(client.selection);
        const e = jsonata(expression);
        const result = e.evaluate(client.response);
        chai_1.expect(result).to.not.be.an('undefined');
        runner.store[storeName] = result;
        return result;
    }),
    regexMatcher_1.regexMatcher(/^I store (?:"([^"]+)" of )?the GQL operation result as "([^"]+)"$/)(async ([expression, storeName], _, runner) => {
        const { appSyncClient: client } = runner.store;
        let result = client.response.data[client.selection];
        if (expression) {
            const e = jsonata(expression);
            result = e.evaluate(result);
        }
        chai_1.expect(result).to.not.be.an('undefined');
        runner.store[storeName] = result;
        return result;
    }),
    regexMatcher_1.regexMatcher(/^(?:"([^"]+)" of )?the GQL operation result (parsed as JSON )?should (equal|match) this JSON$/)(async ([exp, parseAsJson, equalOrMatch], step, runner) => {
        const { appSyncClient: client } = runner.store;
        chai_1.expect(client.response).to.have.property('data');
        chai_1.expect(client.response.data).to.have.property(client.selection);
        if (!step.interpolatedArgument) {
            throw new Error('Must provide argument!');
        }
        const j = JSON.parse(step.interpolatedArgument);
        const opResult = client.response.data[client.selection];
        let fragment = exp ? jsonata(exp).evaluate(opResult) : opResult;
        if (parseAsJson) {
            fragment = JSON.parse(fragment);
        }
        if (equalOrMatch === 'match') {
            chai_1.expect(fragment).to.containSubset(j);
        }
        else {
            chai_1.expect(fragment).to.deep.equal(j);
        }
        return opResult;
    }),
    regexMatcher_1.regexMatcher(/^(?:"([^"]+)" of )?the GQL operation result should (?:(not) )?equal "([^"]+)"$/)(async ([exp, not, expected], _, runner) => {
        const { appSyncClient: client } = runner.store;
        chai_1.expect(client.response).to.have.property('data');
        chai_1.expect(client.response.data).to.have.property(client.selection);
        const opResult = client.response.data[client.selection];
        const fragment = exp ? jsonata(exp).evaluate(opResult) : opResult;
        if (not) {
            chai_1.expect(fragment).to.not.equal(expected);
        }
        else {
            chai_1.expect(fragment).to.equal(expected);
        }
        return opResult;
    }),
    regexMatcher_1.regexMatcher(/^"([^"]+)" of the GQL operation result should be (true|false|null|undefined)$/)(async ([exp, expected], _, runner) => {
        const { appSyncClient: client } = runner.store;
        chai_1.expect(client.response).to.have.property('data');
        chai_1.expect(client.response.data).to.have.property(client.selection);
        const opResult = client.response.data[client.selection];
        const checks = {
            null: fragment => chai_1.expect(fragment).to.equal(null),
            undefined: fragment => chai_1.expect(fragment).to.equal(undefined),
            true: fragment => chai_1.expect(fragment).to.equal(true),
            false: fragment => chai_1.expect(fragment).to.equal(false),
        };
        checks[expected](jsonata(exp).evaluate(opResult));
        return opResult;
    }),
    regexMatcher_1.regexMatcher(/^the GQL queries are authenticated with Cognito$/)(async (_, __, runner) => {
        const { appSyncClient: client } = runner.store;
        client.authorization = 'IAM';
    }),
    regexMatcher_1.regexMatcher(/^the GQL queries are authenticated with the API key "([^"]+)"$/)(async ([apiKey], __, runner) => {
        const { appSyncClient: client } = runner.store;
        client.authorization = 'API_KEY';
        client.apiKey = apiKey;
    }),
    regexMatcher_1.regexMatcher(/^I set the GQL variable "([^"]+)" to "([^"]+)"$/)(async ([name, value], _, runner) => {
        const { appSyncClient: client } = runner.store;
        client.variables[name] = value;
    }),
    regexMatcher_1.regexMatcher(/^I set the GQL variable "([^"]+)" to (the stringified version of )?this JSON$/)(async ([name, stringify], step, runner) => {
        const { appSyncClient: client } = runner.store;
        if (!step.interpolatedArgument) {
            throw new Error('Must provide argument!');
        }
        const j = JSON.parse(step.interpolatedArgument);
        client.variables[name] = stringify ? JSON.stringify(j) : j;
        return client.variables[name];
    }),
    regexMatcher_1.regexMatcher(/^I am subscribed to the "([^"]+)" GQL subscription(?: as "([^"]+)")?( with these variables)?$/)(async ([subscriptionId, userId, withVariables], step, runner) => {
        const { appSyncClient: client } = runner.store;
        let key = subscriptionId;
        if (userId) {
            key = `${key}:${userId}`;
        }
        let variables;
        if (withVariables) {
            if (!step.interpolatedArgument) {
                throw new Error('Must provide argument!');
            }
            variables = JSON.parse(step.interpolatedArgument);
        }
        const query = getQuery(runner.store, client, userId);
        client.subscriptions[key] = await subscribe_1.subscribe(runner, client.subscriptionQueries[subscriptionId], query, variables);
    }),
    regexMatcher_1.regexMatcher(/^I register a "([^"]+)" listener on the "([^"]+)" GQL subscription(?: as "([^"]+)")?( that matches this JSON)?$/)(async ([listener, subscriptionId, userId, matchJSON], step, runner) => {
        const { appSyncClient: client } = runner.store;
        if (client.listenerSubscription[listener]) {
            console.warn(`The listener "${listener}" is already registered!`);
        }
        let key = subscriptionId;
        if (userId) {
            key = `${key}:${userId}`;
        }
        if (!client.subscriptions[key]) {
            throw new Error(`Subscription "${key}" not found!`);
        }
        let matcher = {};
        if (matchJSON) {
            if (!step.interpolatedArgument) {
                throw new Error('Must provide argument!');
            }
            matcher = JSON.parse(step.interpolatedArgument);
        }
        client.subscriptions[key].addListener(listener, matcher);
        client.listenerSubscription[listener] = client.subscriptions[key];
    }),
    regexMatcher_1.regexMatcher(/^I should receive a message on the "([^"]+)" GQL subscription listener?$/)(async ([listener], _, runner) => {
        const { appSyncClient: client } = runner.store;
        if (!client.listenerSubscription[listener]) {
            throw new Error(`Listener "${listener}" not found!`);
        }
        return client.listenerSubscription[listener].listenerMessage(listener);
    }),
    regexMatcher_1.regexMatcher(/^"([^"]+)" of the last "([^"]+)" GQL subscription listener message should equal "([^"]+)"$/)(async ([exp, subscriptionId, expected], _, runner) => {
        const { appSyncClient: client } = runner.store;
        const message = client.subscriptionMessages[subscriptionId][client.subscriptionMessages[subscriptionId].length - 1];
        chai_1.expect(jsonata(exp).evaluate(message)).to.equal(expected);
    }),
];
